package rule

import (
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/db/model"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/enum"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	zhios_order_relate_utils "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/cache"
	zhios_order_relate_logx "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/utils/logx"
	"errors"
	"github.com/shopspring/decimal"
	"time"
	"xorm.io/xorm"
)

func InitForGreenCoinDoubleChainIntegral(redisAddr string) (err error) {
	if redisAddr != "" {
		cache.NewRedis(redisAddr)
	}
	_, err = cache.SelectDb(md.RedisDataBase)
	return
}

func DealUserGreenCoinDoubleChainIntegral(Db *xorm.Engine, uid int, amount, ordId, masterId string) (isExchange bool, err error) {
	user, err := db.DbsUserFindByUid(Db, uid)
	if err != nil {
		return
	}
	if user.Level >= 2 {
		//TODO::暂时写死（至少是银卡会员等级才能兑换）
		return
	}

	amountValue, _ := decimal.NewFromString(amount)
	session := Db.NewSession()
	defer func() {
		session.Close()
		if err := recover(); err != nil {
			_ = zhios_order_relate_logx.Error(err)
		}
	}()
	session.Begin()
	//1、查找 `green_coin_double_chain` 基础设置
	greenCoinDoubleChain, err := db.GreenCoinDoubleChainGetOneByParams(session, map[string]interface{}{
		"key":   "is_use",
		"value": 1,
	})
	if err != nil {
		return
	}

	//2、查找对应直推上级用户
	parentUserRelate, err := db.GetUserParentUserRelate(Db, uid)
	if err != nil {
		return
	}
	if parentUserRelate == nil {
		err = errors.New("未查询到直推上级")
		return
	}

	//3、查询用户和对应上级的虚拟币余额
	userAmount, err := svc.GetUserCoinAmount(session, masterId, greenCoinDoubleChain.Coin2, uid)
	if err != nil {
		return
	}
	parentAmount, err := svc.GetUserCoinAmount(session, masterId, greenCoinDoubleChain.Coin1, parentUserRelate.ParentUid)
	if err != nil {
		return
	}
	userAmountValue, _ := decimal.NewFromString(userAmount)
	parentAmountValue, _ := decimal.NewFromString(parentAmount)
	if parentAmountValue.LessThan(amountValue) {
		//TODO::上级绿色积分小于当前需要兑换积分金额
		return
	}
	if userAmountValue.LessThan(amountValue) {
		err = errors.New("当前用户金额有误，不足以进行兑换")
		return
	}

	//4、查询上级直推的所有下级用户
	sonUserRelates, err := db.DbsUserRelateByParentUid(Db, parentUserRelate.ParentUid, 1)
	if err != nil {
		return
	}
	if sonUserRelates == nil {
		err = errors.New("未查询到直推下级")
		return
	}
	var sonUserIds []int
	for _, v := range *sonUserRelates {
		if v.Uid != uid {
			sonUserIds = append(sonUserIds, v.Uid)
		}
	}

	users, err := db.DbsUserFindByIds(Db, sonUserIds)
	if err != nil {
		return
	}
	var userIds []int
	for _, v := range *users {
		if v.Level >= 2 {
			//TODO::暂时写死（至少是银卡会员等级才能兑换）
			userIds = append(userIds, v.Uid)
		}
	}

	//5、查询上级直推的所有下级用户的虚拟币金额是否有大于当前用户的
	var sonUserVirtualWallet model.UserVirtualAmount
	isHas, err := session.Where(" coin_id = ? and amount >=?", greenCoinDoubleChain.Coin2, amount).In("uid", userIds).Get(&sonUserVirtualWallet)
	if err != nil {
		return
	}
	if isHas == false {
		//TODO::没有符合匹配的下级用户
		return
	}

	/*
		TODO::6、进行“数据变更”
	*/
	//6.1、上级减少amount 绿色积分
	err = DealUserCoin(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          masterId,
		Title:        md.GreenCoinDoubleChainExchangeByCoin1,
		TransferType: md.GreenCoinDoubleChainExchangeByCoin1ForForUserVirtualCoinFlow,
		OrdId:        ordId,
		CoinId:       greenCoinDoubleChain.Coin1,
		Uid:          parentUserRelate.ParentUid,
		Amount:       zhios_order_relate_utils.StrToFloat64(amount),
	})
	if err != nil {
		session.Rollback()
		return
	}
	//6.2、用户减少amount 贡献积分
	err = DealUserCoin(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          masterId,
		Title:        md.GreenCoinDoubleChainExchangeByCoin2,
		TransferType: md.GreenCoinDoubleChainExchangeByCoin2ForForUserVirtualCoinFlow,
		OrdId:        ordId,
		CoinId:       greenCoinDoubleChain.Coin2,
		Uid:          uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(amount),
	})
	//6.3、另一下级减少amount 贡献积分
	err = DealUserCoin(session, md.DealUserCoinReq{
		Kind:         "sub",
		Mid:          masterId,
		Title:        md.GreenCoinDoubleChainExchangeByCoin2,
		TransferType: md.GreenCoinDoubleChainExchangeByCoin2ForForUserVirtualCoinFlow,
		OrdId:        ordId,
		CoinId:       greenCoinDoubleChain.Coin2,
		Uid:          sonUserVirtualWallet.Uid,
		Amount:       zhios_order_relate_utils.StrToFloat64(amount),
	})
	if err != nil {
		session.Rollback()
		return
	}
	//6.4上级余额增加 amount
	var coin model.VirtualCoin
	_, err = session.Where("id = ?", greenCoinDoubleChain.Coin1).Get(&coin)
	if err != nil {
		session.Rollback()
		return
	}
	coinExchangeRatioValue, _ := decimal.NewFromString(coin.ExchangeRatio)
	exchangeAmount, _ := amountValue.Div(coinExchangeRatioValue).Float64()
	orderType := enum.FinUserFlowOrderActionString(md.GreenCoinDoubleChainExchangeForBalanceForFinUserFlow)
	err = svc.DealUserAmount(session, md.DealUserAmount{
		Kind:      "add",
		Mid:       masterId,
		Title:     md.GreenCoinDoubleChainExchangeForBalanceTitleForFinUserFlow,
		OrderType: orderType,
		OrdAction: md.GreenCoinDoubleChainExchangeForBalanceForFinUserFlow,
		OrdId:     ordId,
		Uid:       parentUserRelate.ParentUid,
		Amount:    exchangeAmount,
		Num:       "",
	})
	if err != nil {
		session.Rollback()
		return
	}

	//7、新增 green_coin_double_chain_exchange_records 记录
	afterUserCoinAmount, err := svc.GetUserCoinAmount(session, masterId, greenCoinDoubleChain.Coin1, parentUserRelate.ParentUid)
	if err != nil {
		session.Rollback()
		return
	}
	afterCoinAmountContributeUser1, err := svc.GetUserCoinAmount(session, masterId, greenCoinDoubleChain.Coin2, uid)
	if err != nil {
		session.Rollback()
		return
	}
	afterCoinAmountContributeUser2, err := svc.GetUserCoinAmount(session, masterId, greenCoinDoubleChain.Coin2, sonUserVirtualWallet.Uid)
	if err != nil {
		session.Rollback()
		return
	}
	now := time.Now()
	_, err = db.GreenCoinDoubleChainExchangeRecordsInsert(Db, &model.GreenCoinDoubleChainExchangeRecords{
		Uid:                             parentUserRelate.ParentUid,
		ContributeUid1:                  uid,
		ContributeUid2:                  sonUserVirtualWallet.Uid,
		CoinId1:                         greenCoinDoubleChain.Coin1,
		CoinId2:                         greenCoinDoubleChain.Coin2,
		Amount:                          amount,
		BeforeUserCoinAmount:            parentAmount,
		AfterUserCoinAmount:             afterUserCoinAmount,
		BeforeCoinAmountContributeUser1: userAmount,
		AfterCoinAmountContributeUser1:  afterCoinAmountContributeUser1,
		BeforeCoinAmountContributeUser2: sonUserVirtualWallet.Amount,
		AfterCoinAmountContributeUser2:  afterCoinAmountContributeUser2,
		CreateAt:                        now.Format("2006-01-02 15:04:05"),
		UpdateAt:                        now.Format("2006-01-02 15:04:05"),
	})
	if err != nil {
		session.Rollback()
		return
	}

	session.Commit()
	isExchange = true
	return
}
